@@ -1,5 +1,4 @@
 import functools
-import inspect
 import logging
 from collections import OrderedDict
 from copy import deepcopy
@@ -156,7 +155,7 @@ class Resource(metaclass=DeclarativeMetaclass):
             if f == field:
                 return field_name
         raise AttributeError(
-            "Field %s does not exists in %s resource" % (field, self.__class__)
+            f"Field {field} does not exists in {self.__class__} resource"
         )
 
     def init_instance(self, row=None):
@@ -207,18 +206,15 @@ class Resource(metaclass=DeclarativeMetaclass):
         """
         Creates objects by calling ``bulk_create``.
         """
-        try:
-            if len(self.create_instances) > 0:
-                if not using_transactions and dry_run:
-                    pass
-                else:
-                    self._meta.model.objects.bulk_create(
-                        self.create_instances, batch_size=batch_size
-                    )
-        except Exception as e:
-            self.handle_import_error(result, e, raise_errors)
-        finally:
-            self.create_instances.clear()
+        if len(self.create_instances) > 0 and (using_transactions or not dry_run):
+            try:
+                self._meta.model.objects.bulk_create(
+                    self.create_instances, batch_size=batch_size
+                )
+            except Exception as e:
+                self.handle_import_error(result, e, raise_errors)
+            finally:
+                self.create_instances.clear()
 
     def bulk_update(
         self, using_transactions, dry_run, raise_errors, batch_size=None, result=None
@@ -226,37 +222,31 @@ class Resource(metaclass=DeclarativeMetaclass):
         """
         Updates objects by calling ``bulk_update``.
         """
-        try:
-            if len(self.update_instances) > 0:
-                if not using_transactions and dry_run:
-                    pass
-                else:
-                    self._meta.model.objects.bulk_update(
-                        self.update_instances,
-                        self.get_bulk_update_fields(),
-                        batch_size=batch_size,
-                    )
-        except Exception as e:
-            self.handle_import_error(result, e, raise_errors)
-        finally:
-            self.update_instances.clear()
+        if len(self.update_instances) > 0 and (using_transactions or not dry_run):
+            try:
+                self._meta.model.objects.bulk_update(
+                    self.update_instances,
+                    self.get_bulk_update_fields(),
+                    batch_size=batch_size,
+                )
+            except Exception as e:
+                self.handle_import_error(result, e, raise_errors)
+            finally:
+                self.update_instances.clear()
 
     def bulk_delete(self, using_transactions, dry_run, raise_errors, result=None):
         """
         Deletes objects by filtering on a list of instances to be deleted,
         then calling ``delete()`` on the entire queryset.
         """
-        try:
-            if len(self.delete_instances) > 0:
-                if not using_transactions and dry_run:
-                    pass
-                else:
-                    delete_ids = [o.pk for o in self.delete_instances]
-                    self._meta.model.objects.filter(pk__in=delete_ids).delete()
-        except Exception as e:
-            self.handle_import_error(result, e, raise_errors)
-        finally:
-            self.delete_instances.clear()
+        if len(self.delete_instances) > 0 and (using_transactions or not dry_run):
+            try:
+                delete_ids = [o.pk for o in self.delete_instances]
+                self._meta.model.objects.filter(pk__in=delete_ids).delete()
+            except Exception as e:
+                self.handle_import_error(result, e, raise_errors)
+            finally:
+                self.delete_instances.clear()
 
     def validate_instance(
         self, instance, import_validation_errors=None, validate_unique=True
@@ -434,9 +424,11 @@ class Resource(metaclass=DeclarativeMetaclass):
 
     def get_import_fields(self):
         import_fields = []
+        missing = object()
         for field_name in self.get_import_order():
-            if field_name in self.fields:
-                import_fields.append(self.fields[field_name])
+            field = self.fields.get(field_name, missing)
+            if field is not missing:
+                import_fields.append(field)
                 continue
             # issue 1815
             # allow for fields to be referenced by column_name in `fields` list
@@ -781,7 +773,7 @@ class Resource(metaclass=DeclarativeMetaclass):
             # There is no point logging a transaction error for each row
             # when only the original error is likely to be relevant
             if not isinstance(e, TransactionManagementError):
-                logger.debug(e, exc_info=e)
+                logger.debug(e, exc_info=True)
             row_result.errors.append(
                 self.get_error_result_class()(e, row=row, number=kwargs["row_number"])
             )
@@ -1023,7 +1015,7 @@ class Resource(metaclass=DeclarativeMetaclass):
         r"""
         Override to filter an export queryset.
 
-        :param queryset: The queryset for export (optional).
+        :param queryset: The queryset for export.
 
         :param \**kwargs:
             Metadata which may be associated with the export.
@@ -1032,36 +1024,37 @@ class Resource(metaclass=DeclarativeMetaclass):
         """
         return queryset
 
-    def export_field(self, field, instance):
+    def export_field(self, field, instance, **kwargs):
         field_name = self.get_field_name(field)
         dehydrate_method = field.get_dehydrate_method(field_name)
 
-        method = getattr(self, dehydrate_method, None)
+        if callable(dehydrate_method):
+            method = dehydrate_method
+        else:
+            method = getattr(self, dehydrate_method, None)
+
         if method is not None:
             return method(instance)
-        return field.export(instance)
+        return field.export(instance, **kwargs)
 
-    def get_export_fields(self):
+    def get_export_fields(self, selected_fields=None):
+        fields_ = selected_fields if selected_fields else self.fields
         export_fields = []
-        for field_name in self.get_export_order():
-            if field_name in self.fields:
-                export_fields.append(self.fields[field_name])
-                continue
-            # issue 1828
-            # allow for fields to be referenced by column_name in `fields` list
-            for field in self.fields.values():
-                if field.column_name == field_name:
+        export_order = self.get_export_order()
+        for field_name in export_order:
+            if field_name in fields_:
+                field = self._select_field(field_name)
+                if field is not None:
                     export_fields.append(field)
-                    continue
         return export_fields
 
-    def export_resource(self, instance, fields=None):
-        export_fields = self._get_enabled_export_fields(fields)
-        return [self.export_field(field, instance) for field in export_fields]
+    def export_resource(self, instance, selected_fields=None, **kwargs):
+        export_fields = self.get_export_fields(selected_fields)
+        return [self.export_field(field, instance, **kwargs) for field in export_fields]
 
-    def get_export_headers(self, fields=None):
-        export_fields = self._get_enabled_export_fields(fields)
-        return [force_str(field.column_name) for field in export_fields]
+    def get_export_headers(self, selected_fields=None):
+        export_fields = self.get_export_fields(selected_fields)
+        return [force_str(field.column_name) for field in export_fields if field]
 
     def get_user_visible_fields(self):
         return self.get_import_fields()
@@ -1097,17 +1090,31 @@ class Resource(metaclass=DeclarativeMetaclass):
             queryset = self.get_queryset()
         queryset = self.filter_export(queryset, **kwargs)
         export_fields = kwargs.get("export_fields", None)
-        headers = self.get_export_headers(fields=export_fields)
+        headers = self.get_export_headers(selected_fields=export_fields)
         dataset = tablib.Dataset(headers=headers)
 
         for obj in self.iter_queryset(queryset):
-            r = self.export_resource(obj, fields=export_fields)
+            r = self.export_resource(obj, selected_fields=export_fields, **kwargs)
             dataset.append(r)
 
         self.after_export(queryset, dataset, **kwargs)
 
         return dataset
 
+    def _select_field(self, target_field_name):
+        # select field from fields based on either declared name or column name
+        missing = object()
+        field = self.fields.get(target_field_name, missing)
+        if field is not missing:
+            return field
+
+        for field_name, field in self.fields.items():
+            if target_field_name == field.column_name:
+                return field
+        # it should have been possible to identify the declared field
+        # but warn if not
+        warn(f"cannot identify field for export with name '{target_field_name}'")
+
     def _get_ordered_field_names(self, order_field):
         """
         Return a list of field names, respecting any defined ordering.
@@ -1177,18 +1184,6 @@ class Resource(metaclass=DeclarativeMetaclass):
                 )
             )
 
-    def _get_enabled_export_fields(self, fields_):
-        export_fields = self.get_export_fields()
-
-        if isinstance(fields_, list) and fields_:
-            return [
-                field
-                for field in export_fields
-                if field.attribute in fields_ or field.column_name in fields_
-            ]
-
-        return export_fields
-
 
 class ModelResource(Resource, metaclass=ModelDeclarativeMetaclass):
     """
@@ -1273,7 +1268,7 @@ class ModelResource(Resource, metaclass=ModelDeclarativeMetaclass):
             # The field class may be in a third party library as a subclass
             # of a standard field class.
             # iterate base classes to determine the correct widget class to use.
-            for base_class in inspect.getmro(f.__class__):
+            for base_class in f.__class__.__mro__:
                 if base_class.__name__ in cls.WIDGETS_MAP:
                     result = cls.WIDGETS_MAP[base_class.__name__]
                     if isinstance(result, str):
@@ -1379,18 +1374,72 @@ class ModelResource(Resource, metaclass=ModelDeclarativeMetaclass):
         return cls.__name__
 
 
-def modelresource_factory(model, resource_class=ModelResource):
+def modelresource_factory(
+    model,
+    resource_class=ModelResource,
+    meta_options=None,
+    custom_fields=None,
+    dehydrate_methods=None,
+):
     """
     Factory for creating ``ModelResource`` class for given Django model.
+    This factory function creates a ``ModelResource`` class dynamically, with support
+    for custom fields, methods.
+
+    :param model: Django model class
+
+    :param resource_class: Base resource class (default: ModelResource)
+
+    :param meta_options: Meta options dictionary
+
+    :param custom_fields: Dictionary mapping field names to Field object
+
+    :param dehydrate_methods: Dictionary mapping field names
+                              to dehydrate method (Callable)
+
+    :returns: ModelResource class
     """
-    attrs = {"model": model}
-    Meta = type(str("Meta"), (object,), attrs)
 
-    class_name = model.__name__ + str("Resource")
+    def _create_dehydrate_func_wrapper(func):
+        def wrapper(self, obj):
+            return func(obj)
 
-    class_attrs = {
+        return wrapper
+
+    if meta_options is None:
+        meta_options = {}
+
+    if custom_fields is None:
+        custom_fields = {}
+
+    if dehydrate_methods is None:
+        dehydrate_methods = {}
+
+    for field_name, field in custom_fields.items():
+        if not isinstance(field, Field):
+            raise ValueError(
+                f"custom_fields['{field_name}'] must be a Field instance, "
+                f"got {type(field).__name__}"
+            )
+
+    meta_class_attrs = {**meta_options, "model": model}
+    Meta = type("Meta", (object,), meta_class_attrs)
+
+    resource_class_name = model.__name__ + "Resource"
+    resource_class_attrs = {
         "Meta": Meta,
     }
+    resource_class_attrs.update(custom_fields)
+
+    for field_name, method in dehydrate_methods.items():
+        if not callable(method):
+            raise ValueError(
+                f"dehydrate_methods['{field_name}'] must be callable, "
+                f"got {type(method).__name__}"
+            )
+
+        method_name = f"dehydrate_{field_name}"
+        resource_class_attrs[method_name] = _create_dehydrate_func_wrapper(method)
 
     metaclass = ModelDeclarativeMetaclass
-    return metaclass(class_name, (resource_class,), class_attrs)
+    return metaclass(resource_class_name, (resource_class,), resource_class_attrs)
